@@ -34,6 +34,7 @@ public:
   QString getDefaultTextColor() { return defaultTextColor; }
   QString getActiveTextColor() { return activeTextColor; }
   QString getSecondaryTextColor() { return secondaryTextColor; }
+  QString getAccentColor() { return accentColor; }
   QString getSystemFontFamily() { return systemFontFamily; }
 
 private:
@@ -41,6 +42,7 @@ private:
   QString defaultTextColor;
   QString activeTextColor;
   QString secondaryTextColor;
+  QString accentColor;
   QString systemFontFamily;
 };
 
