@@ -40,16 +40,21 @@ public:
   bool openExternalPlayer() { return openExternalPlayerFlag; }
 
   bool openExternalBrowser() { return openExternalBrowserFlag; }
+
+  bool hideImages() { return hideImagesFlag; }
   
 private slots:
   void on_playerCheckBox_toggled(bool checked);
   void on_browserCheckBox_toggled(bool checked);
 
+  void on_imagesCheckBox_toggled(bool checked);
+
 private:
   Ui::LinPreferencesForm *ui;
 
   bool openExternalPlayerFlag;
   bool openExternalBrowserFlag;
+  bool hideImagesFlag;
 };
 
 #endif // LINPREFERENCESFORM_H
